[PATCH] drm: rcar_lvds: Fix dual link mode operations

Jacopo Mondi jacopo+renesas at jmondi.org
Tue Jul 23 16:57:00 UTC 2019


The R-Car LVDS encoder units support dual-link operations by splitting
the pixel output between the primary encoder and the companion one.

In order for the primary encoder to succesfully control the companion's
operations this should not fail at probe time and register itself its
associated drm bridge so that the primary one can find it.

Currently the companion encoder fails at probe time, causing the
registration of the primary to fail preventing the whole DU unit to be
registered correctly.

Fixes: fa440d870358 ("drm: rcar-du: lvds: Add support for dual-link mode")
Reported-by: Fabrizio Castro <fabrizio.castro at bp.renesas.com>
Signed-off-by: Jacopo Mondi <jacopo+renesas at jmondi.org>

---
The "Fixes" tag refers to a patch currently part of the
renesas-drivers-2019-07-09-v5.2 branch of Geert's renesas-drivers tree.

 drivers/gpu/drm/rcar-du/rcar_lvds.c | 31 +++++++++++++++++++++--------
 1 file changed, 23 insertions(+), 8 deletions(-)

diff --git a/drivers/gpu/drm/rcar-du/rcar_lvds.c b/drivers/gpu/drm/rcar-du/rcar_lvds.c
index bada7ee98544..8b015ba95895 100644
--- a/drivers/gpu/drm/rcar-du/rcar_lvds.c
+++ b/drivers/gpu/drm/rcar-du/rcar_lvds.c
@@ -767,14 +767,29 @@ static int rcar_lvds_parse_dt(struct rcar_lvds *lvds)
 	of_node_put(remote_input);
 	of_node_put(remote);

-	/*
-	 * On D3/E3 the LVDS encoder provides a clock to the DU, which can be
-	 * used for the DPAD output even when the LVDS output is not connected.
-	 * Don't fail probe in that case as the DU will need the bridge to
-	 * control the clock.
-	 */
-	if (lvds->info->quirks & RCAR_LVDS_QUIRK_EXT_PLL)
-		return ret == -ENODEV ? 0 : ret;
+	switch (ret) {
+	case -ENODEV:
+		/*
+		 * On D3/E3 the LVDS encoder provides a clock to the DU, which
+		 * can be used for the DPAD output even when the LVDS output is
+		 * not connected. Don't fail probe in that case as the DU will
+		 * need the bridge to control the clock.
+		 */
+		if (lvds->info->quirks & RCAR_LVDS_QUIRK_EXT_PLL)
+			ret = 0;
+		break;
+	case -ENXIO:
+		/*
+		 * When the LVDS output is used in dual link mode, the
+		 * companion encoder fails at
+		 * 'rcar_lvds_parse_dt_companion()'. Don't fail probe in
+		 * that case as the master encoder will need the companion's
+		 * bridge to control its operations.
+		 */
+		if (lvds->info->quirks & RCAR_LVDS_QUIRK_DUAL_LINK)
+			ret = 0;
+		break;
+	}

 	return ret;
 }
--
2.22.0



More information about the dri-devel mailing list